@@ -1,7 +1,7 @@
 /*
 libdmtx - Data Matrix Encoding/Decoding Library
 
-Copyright (c) 2008 Mike Laughton
+Copyright (C) 2008, 2009 Mike Laughton
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxencode.c 514 2008-11-19 17:10:44Z mblaughton $ */
+/* $Id: dmtxencode.c 763 2009-02-26 23:47:43Z mblaughton $ */
 
 /**
  * @file dmtxencode.c
@@ -69,16 +69,22 @@ Contact: mike@dragonflylogic.com
  * @brief  Initialize encode struct with default values
  * @return Initialized DmtxEncode struct
  */
-extern DmtxEncode
-dmtxEncodeStructInit(void)
+extern DmtxEncode *
+dmtxEncodeCreate(void)
 {
-   DmtxEncode enc;
+   DmtxEncode *enc;
 
-   memset(&enc, 0x00, sizeof(DmtxEncode));
+   enc = (DmtxEncode *)calloc(1, sizeof(DmtxEncode));
+   if(enc == NULL)
+      return NULL;
 
-   enc.scheme = DmtxSchemeEncodeAscii;
-   enc.moduleSize = 5;
-   enc.marginSize = 10;
+   enc->scheme = DmtxSchemeEncodeAscii;
+   enc->sizeIdxRequest = DmtxSymbolSquareAuto;
+   enc->marginSize = 10;
+   enc->moduleSize = 5;
+   enc->pixelPacking = DmtxPack24bppRGB;
+   enc->imageFlip = DmtxFlipNone;
+   enc->rowPadBytes = 0;
 
    /* Initialize background color to white */
 /* enc.region.gradient.ray.p.R = 255.0;
@@ -87,12 +93,9 @@ dmtxEncodeStructInit(void)
 
    /* Initialize foreground color to black */
 /* enc.region.gradient.tMin = 0.0;
-   enc.region.gradient.tMax = dmtxColor3Mag(&(enc.region.gradient.ray.p)); */
+   enc.region.gradient.tMax = xyz; */
 
-/* dmtxColor3Scale(&(enc.region.gradient.ray.c),
-         &(enc.region.gradient.ray.p), -1.0/enc.region.gradient.tMax); */
-
-   dmtxMatrix3Identity(enc.xfrm);
+   dmtxMatrix3Identity(enc->xfrm);
 
    return enc;
 }
@@ -102,16 +105,87 @@ dmtxEncodeStructInit(void)
  * @param  enc
  * @return void
  */
-extern void
-dmtxEncodeStructDeInit(DmtxEncode *enc)
+extern DmtxPassFail
+dmtxEncodeDestroy(DmtxEncode **enc)
 {
-   if(enc == NULL)
-      return;
+   if(enc == NULL || *enc == NULL)
+      return DmtxFail;
 
-   dmtxImageFree(&(enc->image));
-   dmtxMessageFree(&(enc->message));
+   dmtxImageDestroy(&((*enc)->image));
+   dmtxMessageDestroy(&((*enc)->message));
 
-   memset(enc, 0x00, sizeof(DmtxEncode));
+   free(*enc);
+
+   *enc = NULL;
+
+   return DmtxPass;
+}
+
+/**
+ * @brief  Set encoding behavior property
+ * @param  enc
+ * @param  prop
+ * @param  value
+ * @return DmtxPass | DmtxFail
+ */
+extern DmtxPassFail
+dmtxEncodeSetProp(DmtxEncode *enc, int prop, int value)
+{
+   switch(prop) {
+
+      /* Encoding details */
+      case DmtxPropScheme:
+         enc->scheme = value;
+         break;
+      case DmtxPropSizeRequest:
+         enc->sizeIdxRequest = value;
+         break;
+
+      /* Presentation details */
+      case DmtxPropMarginSize:
+         enc->marginSize = value;
+         break;
+      case DmtxPropModuleSize:
+         enc->moduleSize = value;
+         break;
+
+      /* Image properties */
+      case DmtxPropPixelPacking:
+         enc->pixelPacking = value;
+         break;
+      case DmtxPropImageFlip:
+         enc->imageFlip = value;
+         break;
+      case DmtxPropRowPadBytes:
+         enc->rowPadBytes = value;
+      default:
+         break;
+   }
+
+   return DmtxPass;
+}
+
+/**
+ * @brief  Get encoding behavior property
+ * @param  enc
+ * @param  prop
+ * @return value
+ */
+extern int
+dmtxEncodeGetProp(DmtxEncode *enc, int prop)
+{
+   switch(prop) {
+      case DmtxPropMarginSize:
+         return enc->marginSize;
+      case DmtxPropModuleSize:
+         return enc->moduleSize;
+      case DmtxPropScheme:
+         return enc->scheme;
+      default:
+         break;
+   }
+
+   return DmtxUndefined;
 }
 
 /**
@@ -120,26 +194,30 @@ dmtxEncodeStructDeInit(DmtxEncode *enc)
  * @param  inputSize
  * @param  inputString
  * @param  sizeIdxRequest
- * @return DMTX_SUCCESS | DMTX_FAILURE
+ * @return DmtxPass | DmtxFail
  */
-extern int
-dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString, int sizeIdxRequest)
+extern DmtxPassFail
+dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString)
 {
    int dataWordCount;
    int sizeIdx;
+   int padCount;
+   int width, height, bitsPerPixel;
    unsigned char buf[4096];
+   unsigned char *pxl;
 
    /* Encode input string into data codewords */
-   sizeIdx = sizeIdxRequest;
+   sizeIdx = enc->sizeIdxRequest;
    dataWordCount = EncodeDataCodewords(buf, inputString, inputSize, enc->scheme, &sizeIdx);
    if(dataWordCount <= 0)
-      return(DMTX_FAILURE);
+      return(DmtxFail);
 
    /* EncodeDataCodewords() should have updated any auto sizeIdx to a real one */
    assert(sizeIdx != DmtxSymbolSquareAuto && sizeIdx != DmtxSymbolRectAuto);
 
    /* Add pad characters to match a standard symbol size (whether smallest or requested) */
-   AddPadChars(buf, &dataWordCount, dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx));
+   padCount = AddPadChars(buf, &dataWordCount,
+         dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx));
 
    /* XXX we can remove a lot of this redundant data */
    enc->region.sizeIdx = sizeIdx;
@@ -149,31 +227,44 @@ dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString,
    enc->region.mappingCols = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixCols, sizeIdx);
 
    /* Allocate memory for message and array */
-   enc->message = dmtxMessageMalloc(sizeIdx, DMTX_FORMAT_MATRIX);
+   enc->message = dmtxMessageCreate(sizeIdx, DmtxFormatMatrix);
+   enc->message->padCount = padCount;
    memcpy(enc->message->code, buf, dataWordCount);
 
-/* fprintf(stdout, "\n\nsize:    %dx%d w/ %d error codewords\n", rows, cols, errorWordLength(enc->region.sizeIdx)); */
-
    /* Generate error correction codewords */
    GenReedSolEcc(enc->message, enc->region.sizeIdx);
 
    /* Module placement in region */
-   ModulePlacementEcc200(enc->message->array, enc->message->code, enc->region.sizeIdx, DMTX_MODULE_ON_RGB);
+   ModulePlacementEcc200(enc->message->array, enc->message->code,
+         enc->region.sizeIdx, DmtxModuleOnRGB);
+
+   width = 2 * enc->marginSize + (enc->region.symbolCols * enc->moduleSize);
+   height = 2 * enc->marginSize + (enc->region.symbolRows * enc->moduleSize);
+   bitsPerPixel = GetBitsPerPixel(enc->pixelPacking);
+   if(bitsPerPixel == DmtxUndefined)
+      return DmtxFail;
+   assert(bitsPerPixel % 8 == 0);
 
    /* Allocate memory for the image to be generated */
-   enc->image = dmtxImageMalloc(
-         2 * enc->marginSize + (enc->region.symbolCols * enc->moduleSize),
-         2 * enc->marginSize + (enc->region.symbolRows * enc->moduleSize));
+   pxl = (unsigned char *)malloc(width * height * (bitsPerPixel/8) + enc->rowPadBytes);
+   if(pxl == NULL) {
+      perror("pixel malloc error");
+      return DmtxFail;
+   }
 
+   enc->image = dmtxImageCreate(pxl, width, height, enc->pixelPacking);
    if(enc->image == NULL) {
       perror("image malloc error");
-      return DMTX_FAILURE;
+      return DmtxFail;
    }
 
+   dmtxImageSetProp(enc->image, DmtxPropImageFlip, enc->imageFlip);
+   dmtxImageSetProp(enc->image, DmtxPropRowPadBytes, enc->rowPadBytes);
+
    /* Insert finder and aligment pattern modules */
    PrintPattern(enc);
 
-   return DMTX_SUCCESS;
+   return DmtxPass;
 }
 
 /**
@@ -182,16 +273,16 @@ dmtxEncodeDataMatrix(DmtxEncode *enc, int inputSize, unsigned char *inputString,
  * @param  inputSize
  * @param  inputString
  * @param  sizeIdxRequest
- * @return DMTX_SUCCESS | DMTX_FAILURE
+ * @return DmtxPass | DmtxFail
  */
-extern int
-dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString, int sizeIdxRequest)
+extern DmtxPassFail
+dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString)
 {
    int dataWordCount;
    int tmpInputSize;
    unsigned char *inputStart;
    int splitInputSize[3];
-   int sizeIdx;
+   int sizeIdx, sizeIdxRequest;
    int splitSizeIdxAttempt, splitSizeIdxFirst, splitSizeIdxLast;
    unsigned char buf[3][4096];
    DmtxEncode encGreen, encBlue;
@@ -207,15 +298,12 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
     * 6) take the 3 different images you created and write out a new barcode
     */
 
-   /* XXX we're going to force ascii until we fix the problem with C40/Text termination */
-   enc->scheme = DmtxSchemeEncodeAscii;
-
    /* Encode full input string to establish baseline data codeword count */
-   sizeIdx = sizeIdxRequest;
+   sizeIdx = sizeIdxRequest = enc->sizeIdxRequest;
    /* XXX buf can be changed here to use all 3 buffers' length */
    dataWordCount = EncodeDataCodewords(buf[0], inputString, inputSize, enc->scheme, &sizeIdx);
    if(dataWordCount <= 0)
-      return DMTX_FAILURE;
+      return DmtxFail;
 
    /* Use 1/3 (ceiling) of inputSize establish input size target */
    tmpInputSize = (inputSize + 2) / 3;
@@ -226,14 +314,14 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
 
    /* Use 1/3 (floor) of dataWordCount establish first symbol size attempt */
    splitSizeIdxFirst = FindCorrectSymbolSize(tmpInputSize, sizeIdxRequest);
-   if(splitSizeIdxFirst == -1)
-      return DMTX_FAILURE;
+   if(splitSizeIdxFirst == DmtxUndefined)
+      return DmtxFail;
 
    /* Set the last possible symbol size for this symbol shape or specific size request */
    if(sizeIdxRequest == DmtxSymbolSquareAuto)
-      splitSizeIdxLast = DMTX_SYMBOL_SQUARE_COUNT - 1;
+      splitSizeIdxLast = DmtxSymbolSquareCount - 1;
    else if(sizeIdxRequest == DmtxSymbolRectAuto)
-      splitSizeIdxLast = DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT - 1;
+      splitSizeIdxLast = DmtxSymbolSquareCount + DmtxSymbolRectCount - 1;
    else
       splitSizeIdxLast = splitSizeIdxFirst;
 
@@ -270,52 +358,54 @@ dmtxEncodeDataMosaic(DmtxEncode *enc, int inputSize, unsigned char *inputString,
       break;
    }
 
+   dmtxEncodeSetProp(enc, DmtxPropSizeRequest, splitSizeIdxAttempt);
+
    /* Now we have the correct lengths for splitInputSize, and they all fit into the desired size */
    encGreen = *enc;
    encBlue = *enc;
 
    /* First encode red to the main encode struct (image portion will be overwritten) */
    inputStart = inputString;
-   dmtxEncodeDataMatrix(enc, splitInputSize[0], inputStart, splitSizeIdxAttempt);
+   dmtxEncodeDataMatrix(enc, splitInputSize[0], inputStart);
 
    inputStart += splitInputSize[0];
-   dmtxEncodeDataMatrix(&encGreen, splitInputSize[1], inputStart, splitSizeIdxAttempt);
+   dmtxEncodeDataMatrix(&encGreen, splitInputSize[1], inputStart);
 
    inputStart += splitInputSize[1];
-   dmtxEncodeDataMatrix(&encBlue, splitInputSize[2], inputStart, splitSizeIdxAttempt);
+   dmtxEncodeDataMatrix(&encBlue, splitInputSize[2], inputStart);
 
    mappingRows = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixRows, splitSizeIdxAttempt);
    mappingCols = dmtxGetSymbolAttribute(DmtxSymAttribMappingMatrixCols, splitSizeIdxAttempt);
 
    memset(enc->message->array, 0x00, sizeof(unsigned char) * enc->region.mappingRows * enc->region.mappingCols);
-   ModulePlacementEcc200(enc->message->array, enc->message->code, enc->region.sizeIdx, DMTX_MODULE_ON_RED);
+   ModulePlacementEcc200(enc->message->array, enc->message->code, enc->region.sizeIdx, DmtxModuleOnRed);
 
    /* Data Mosaic will traverse this array multiple times -- reset
-      DMTX_MODULE_ASSIGNED and DMX_MODULE_VISITED bits before starting */
+      DmtxModuleAssigned and DMX_MODULE_VISITED bits before starting */
    for(row = 0; row < mappingRows; row++) {
       for(col = 0; col < mappingCols; col++) {
-         enc->message->array[row*mappingCols+col] &= (0xff ^ (DMTX_MODULE_ASSIGNED | DMTX_MODULE_VISITED));
+         enc->message->array[row*mappingCols+col] &= (0xff ^ (DmtxModuleAssigned | DmtxModuleVisited));
       }
    }
 
-   ModulePlacementEcc200(enc->message->array, encGreen.message->code, enc->region.sizeIdx, DMTX_MODULE_ON_GREEN);
+   ModulePlacementEcc200(enc->message->array, encGreen.message->code, enc->region.sizeIdx, DmtxModuleOnGreen);
 
    /* Data Mosaic will traverse this array multiple times -- reset
-      DMTX_MODULE_ASSIGNED and DMX_MODULE_VISITED bits before starting */
+      DmtxModuleAssigned and DMX_MODULE_VISITED bits before starting */
    for(row = 0; row < mappingRows; row++) {
       for(col = 0; col < mappingCols; col++) {
-         enc->message->array[row*mappingCols+col] &= (0xff ^ (DMTX_MODULE_ASSIGNED | DMTX_MODULE_VISITED));
+         enc->message->array[row*mappingCols+col] &= (0xff ^ (DmtxModuleAssigned | DmtxModuleVisited));
       }
    }
 
-   ModulePlacementEcc200(enc->message->array, encBlue.message->code, enc->region.sizeIdx, DMTX_MODULE_ON_BLUE);
+   ModulePlacementEcc200(enc->message->array, encBlue.message->code, enc->region.sizeIdx, DmtxModuleOnBlue);
 
 /* dmtxEncodeStructDeInit(&encGreen);
    dmtxEncodeStructDeInit(&encBlue); */
 
    PrintPattern(enc);
 
-   return DMTX_SUCCESS;
+   return DmtxPass;
 }
 
 /**
@@ -358,7 +448,7 @@ EncodeDataCodewords(unsigned char *buf, unsigned char *inputString,
 
    /* parameter sizeIdx is requested value, returned sizeIdx is decision */
    *sizeIdx = FindCorrectSymbolSize(dataWordCount, *sizeIdx);
-   if(*sizeIdx == -1)
+   if(*sizeIdx == DmtxUndefined)
       return 0;
 
    return dataWordCount;
@@ -371,18 +461,25 @@ EncodeDataCodewords(unsigned char *buf, unsigned char *inputString,
  * @param  paddedSize
  * @return void
  */
-static void
+static int
 AddPadChars(unsigned char *buf,  int *bufSize, int paddedSize)
 {
+   int padCount = 0;
+
    /* First pad character is not randomized */
-   if(*bufSize < paddedSize)
+   if(*bufSize < paddedSize) {
+      padCount++;
       buf[(*bufSize)++] = DMTX_CHAR_ASCII_PAD;
+   }
 
    /* All remaining pad characters are randomized based on character position */
    while(*bufSize < paddedSize) {
+      padCount++;
       buf[*bufSize] = Randomize253State(DMTX_CHAR_ASCII_PAD, *bufSize + 1);
       (*bufSize)++;
    }
+
+   return padCount;
 }
 
 /**
@@ -399,8 +496,12 @@ Randomize253State(unsigned char codewordValue, int codewordPosition)
 
    pseudoRandom = ((149 * codewordPosition) % 253) + 1;
    tmp = codewordValue + pseudoRandom;
+   if(tmp > 254)
+      tmp -= 254;
 
-   return (tmp <= 254) ? tmp : tmp - 254;
+   assert(tmp >= 0 && tmp < 256);
+
+   return (unsigned char)tmp;
 }
 
 /**
@@ -433,8 +534,9 @@ PrintPattern(DmtxEncode *enc)
    int symbolRow, symbolCol;
    int pixelRow, pixelCol;
    int moduleStatus;
+   size_t rowSize, height;
+   int rgb[3];
    double sxy, txy;
-   DmtxRgb rgb;
    DmtxMatrix3 m1, m2;
    DmtxVector2 vIn, vOut;
 
@@ -449,11 +551,10 @@ PrintPattern(DmtxEncode *enc)
    dmtxMatrix3Scale(m2, enc->moduleSize, enc->moduleSize);
    dmtxMatrix3Multiply(enc->rxfrm, m2, m1);
 
-   /* Print raster version of barcode pattern
-      IMPORTANT: DmtxImage is stored with its origin at bottom-right
-      (unlike common image file formats) to preserve "right-handed" 2D space */
+   rowSize = dmtxImageGetProp(enc->image, DmtxPropRowSizeBytes);
+   height = dmtxImageGetProp(enc->image, DmtxPropHeight);
 
-   memset(enc->image->pxl, 0xff, dmtxImageGetProp(enc->image, DmtxPropArea) * sizeof(DmtxRgb));
+   memset(enc->image->pxl, 0xff, rowSize * height);
 
    for(symbolRow = 0; symbolRow < enc->region.symbolRows; symbolRow++) {
       for(symbolCol = 0; symbolCol < enc->region.symbolCols; symbolCol++) {
@@ -466,14 +567,18 @@ PrintPattern(DmtxEncode *enc)
          pixelCol = (int)(vOut.X);
          pixelRow = (int)(vOut.Y);
 
-         moduleStatus = dmtxSymbolModuleStatus(enc->message, enc->region.sizeIdx, symbolRow, symbolCol);
+         moduleStatus = dmtxSymbolModuleStatus(enc->message,
+               enc->region.sizeIdx, symbolRow, symbolCol);
 
          for(i = pixelRow; i < pixelRow + enc->moduleSize; i++) {
             for(j = pixelCol; j < pixelCol + enc->moduleSize; j++) {
-               rgb[0] = ((moduleStatus & DMTX_MODULE_ON_RED) != 0x00) ? 0 : 255;
-               rgb[1] = ((moduleStatus & DMTX_MODULE_ON_GREEN) != 0x00) ? 0 : 255;
-               rgb[2] = ((moduleStatus & DMTX_MODULE_ON_BLUE) != 0x00) ? 0 : 255;
-               dmtxImageSetRgb(enc->image, j, i, rgb);
+               rgb[0] = ((moduleStatus & DmtxModuleOnRed) != 0x00) ? 0 : 255;
+               rgb[1] = ((moduleStatus & DmtxModuleOnGreen) != 0x00) ? 0 : 255;
+               rgb[2] = ((moduleStatus & DmtxModuleOnBlue) != 0x00) ? 0 : 255;
+/*             dmtxImageSetRgb(enc->image, j, i, rgb); */
+               dmtxImageSetPixelValue(enc->image, j, i, 0, rgb[0]);
+               dmtxImageSetPixelValue(enc->image, j, i, 1, rgb[1]);
+               dmtxImageSetPixelValue(enc->image, j, i, 2, rgb[2]);
             }
          }
 
@@ -860,7 +965,9 @@ EncodeTripletCodeword(DmtxChannel *channel)
                channel->inputPtr--;
             }
 
-            inputCount = channel->inputStop - channel->inputPtr;
+            assert(channel->inputStop >= channel->inputPtr);
+            assert(channel->inputStop - channel->inputPtr <= INT_MAX);
+            inputCount = (int)(channel->inputStop - channel->inputPtr);
 
             ProcessEndOfSymbolTriplet(channel, &triplet, tripletCount, inputCount);
             break;
@@ -1298,7 +1405,7 @@ ProcessEndOfSymbolTriplet(DmtxChannel *channel, DmtxTriplet *triplet, int triple
 /* XXX this is broken -- what if someone asks for DmtxSymbolRectAuto or a specific sizeIdx? */
    sizeIdx = FindCorrectSymbolSize(currentByte + ((inputCount == 3) ? 2 : inputCount),
          DmtxSymbolSquareAuto);
-   /* XXX test for sizeIdx == -1 here */
+   /* XXX test for sizeIdx == DmtxUndefined here */
    remainingCodewords = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx) - currentByte;
 
    /* XXX the big problem with all of these special cases is what if one of
@@ -1401,7 +1508,9 @@ TestForEndOfSymbolEdifact(DmtxChannel *channel)
     */
 
    /* Count remaining input values assuming EDIFACT encodation */
-   edifactValues = channel->inputStop - channel->inputPtr;
+   assert(channel->inputStop >= channel->inputPtr);
+   assert(channel->inputStop - channel->inputPtr <= INT_MAX);
+   edifactValues = (int)(channel->inputStop - channel->inputPtr);
 
    /* Can't end symbol right now if there are 5+ values remaining
       (noting that '9999' can still terminate in case (f)) */
@@ -1413,7 +1522,7 @@ TestForEndOfSymbolEdifact(DmtxChannel *channel)
 
    currentByte = channel->currentLength/12;
    sizeIdx = FindCorrectSymbolSize(currentByte, DmtxSymbolSquareAuto);
-   /* XXX test for sizeIdx == -1 here */
+   /* XXX test for sizeIdx == DmtxUndefined here */
    symbolCodewords = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, sizeIdx) - currentByte;
 
    /* Test for special case condition */
