@@ -1,7 +1,7 @@
 /*
 libdmtx - Data Matrix Encoding/Decoding Library
 
-Copyright (c) 2008 Mike Laughton
+Copyright (C) 2008, 2009 Mike Laughton
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxstatic.h 514 2008-11-19 17:10:44Z mblaughton $ */
+/* $Id: dmtxstatic.h 757 2009-02-26 16:00:35Z mblaughton $ */
 
 /**
  * @file dmtxstatic.h
@@ -30,12 +30,13 @@ Contact: mike@dragonflylogic.com
 #ifndef __DMTXSTATIC_H__
 #define __DMTXSTATIC_H__
 
-#define DMTX_ALMOST_ZERO               0.000001
-#define DMTX_ALMOST_INFINITY          -1
+#define DmtxAlmostZero                 0.000001
+#define DmtxAlmostInfinity            -1
 
-#define DMTX_RANGE_GOOD                0
-#define DMTX_RANGE_BAD                 1
-#define DMTX_RANGE_EOF                 2
+#define DmtxC40TextBasicSet            0
+#define DmtxC40TextShift1              1
+#define DmtxC40TextShift2              2
+#define DmtxC40TextShift3              3
 
 #undef min
 #define min(X,Y) (((X) < (Y)) ? (X) : (Y))
@@ -43,12 +44,18 @@ Contact: mike@dragonflylogic.com
 #undef max
 #define max(X,Y) (((X) > (Y)) ? (X) : (Y))
 
+typedef enum {
+   DmtxRangeGood,
+   DmtxRangeBad,
+   DmtxRangeEnd
+} DmtxRange;
+
 typedef enum {
    DmtxEdgeTop    = 0x01 << 0,
    DmtxEdgeBottom = 0x01 << 1,
    DmtxEdgeLeft   = 0x01 << 2,
    DmtxEdgeRight  = 0x01 << 3
-} DmtxEdgeLoc;
+} DmtxEdge;
 
 typedef enum {
    DmtxMaskBit1 = 0x01 << 7,
@@ -59,7 +66,7 @@ typedef enum {
    DmtxMaskBit6 = 0x01 << 2,
    DmtxMaskBit7 = 0x01 << 1,
    DmtxMaskBit8 = 0x01 << 0
-} DmtxBitMask;
+} DmtxMaskBit;
 
 /**
  * @struct DmtxFollow
@@ -92,52 +99,56 @@ typedef struct DmtxBresLine_struct {
    DmtxPixelLoc loc1;
 } DmtxBresLine;
 
+typedef struct C40TextState_struct {
+   int shift;
+   DmtxBoolean upperShift;
+} C40TextState;
+
 /* dmtxregion.c */
 static double RightAngleTrueness(DmtxVector2 c0, DmtxVector2 c1, DmtxVector2 c2, double angle);
 static DmtxPointFlow MatrixRegionSeekEdge(DmtxDecode *dec, DmtxPixelLoc loc0);
-static int MatrixRegionOrientation(DmtxDecode *dec, DmtxRegion *reg, DmtxPointFlow flowBegin);
+static DmtxPassFail MatrixRegionOrientation(DmtxDecode *dec, DmtxRegion *reg, DmtxPointFlow flowBegin);
 static long DistanceSquared(DmtxPixelLoc a, DmtxPixelLoc b);
-static unsigned char *GetCacheAddress(DmtxDecode *dec, int x, int y);
-static int ReadModuleColor(DmtxImage *image, DmtxRegion *region,
-      int symbolRow, int symbolCol, int sizeIdx);
+static int ReadModuleColor(DmtxDecode *dec, DmtxRegion *reg, int symbolRow, int symbolCol, int sizeIdx, int colorPlane);
 
-static int MatrixRegionFindSize(DmtxDecode *dec, DmtxRegion *reg);
-static int CountJumpTally(DmtxImage *img, DmtxRegion *reg, int xStart, int yStart, DmtxDirection dir);
+static DmtxPassFail MatrixRegionFindSize(DmtxDecode *dec, DmtxRegion *reg);
+static int CountJumpTally(DmtxDecode *dec, DmtxRegion *reg, int xStart, int yStart, DmtxDirection dir);
 static DmtxPointFlow GetPointFlow(DmtxDecode *dec, int colorPlane, DmtxPixelLoc loc, int arrive);
 static DmtxPointFlow FindStrongestNeighbor(DmtxDecode *dec, DmtxPointFlow center, int sign);
 static DmtxFollow FollowSeek(DmtxDecode *dec, DmtxRegion *reg, int seek);
 static DmtxFollow FollowSeekLoc(DmtxDecode *dec, DmtxPixelLoc loc);
 static DmtxFollow FollowStep(DmtxDecode *dec, DmtxRegion *reg, DmtxFollow followBeg, int sign);
-static DmtxFollow FollowStep2(DmtxDecode *dec, DmtxRegion *reg, DmtxFollow followBeg, int sign);
-static int TrailBlazeContinuous(DmtxDecode *dec, DmtxRegion *reg, DmtxPointFlow flowBegin, int maxDiagonal);
+static DmtxFollow FollowStep2(DmtxDecode *dec, DmtxFollow followBeg, int sign);
+static DmtxPassFail TrailBlazeContinuous(DmtxDecode *dec, DmtxRegion *reg, DmtxPointFlow flowBegin, int maxDiagonal);
 static int TrailBlazeGapped(DmtxDecode *dec, DmtxRegion *reg, DmtxBresLine line, int streamDir);
 static int TrailClear(DmtxDecode *dec, DmtxRegion *reg, int clearMask);
 static DmtxBestLine FindBestSolidLine(DmtxDecode *dec, DmtxRegion *reg, int step0, int step1, int streamDir, int houghAvoid);
-static DmtxBestLine FindBestSolidLine2(DmtxDecode *dec, DmtxRegion *reg, DmtxPixelLoc loc0, int tripSteps, int sign, int houghAvoid);
-static int FindTravelLimits(DmtxDecode *dec, DmtxRegion *reg, DmtxBestLine *line);
-static int MatrixRegionAlignCalibEdge(DmtxDecode *dec, DmtxRegion *reg, int whichEdge);
+static DmtxBestLine FindBestSolidLine2(DmtxDecode *dec, DmtxPixelLoc loc0, int tripSteps, int sign, int houghAvoid);
+static DmtxPassFail FindTravelLimits(DmtxDecode *dec, DmtxRegion *reg, DmtxBestLine *line);
+static DmtxPassFail MatrixRegionAlignCalibEdge(DmtxDecode *dec, DmtxRegion *reg, int whichEdge);
 static DmtxBresLine BresLineInit(DmtxPixelLoc loc0, DmtxPixelLoc loc1, DmtxPixelLoc locInside);
-static int BresLineGetStep(DmtxBresLine line, DmtxPixelLoc target, int *travel, int *outward);
-static int BresLineStep(DmtxBresLine *line, int travel, int outward);
+static DmtxPassFail BresLineGetStep(DmtxBresLine line, DmtxPixelLoc target, int *travel, int *outward);
+static DmtxPassFail BresLineStep(DmtxBresLine *line, int travel, int outward);
 /*static void WriteDiagnosticImage(DmtxDecode *dec, DmtxRegion *reg, char *imagePath);*/
 
 /* dmtxdecode.c */
-static void DecodeDataStream(DmtxMessage *message, int sizeIdx, unsigned char *outputStart);
+static void DecodeDataStream(DmtxMessage *msg, int sizeIdx, unsigned char *outputStart);
 static unsigned char *NextEncodationScheme(DmtxSchemeDecode *encScheme, unsigned char *ptr);
-static unsigned char *DecodeSchemeAsciiStd(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd);
-static unsigned char *DecodeSchemeAsciiExt(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd);
-static unsigned char *DecodeSchemeC40Text(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd, DmtxSchemeDecode encScheme);
-static unsigned char *DecodeSchemeX12(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd);
-static unsigned char *DecodeSchemeEdifact(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd);
-static unsigned char *DecodeSchemeBase256(DmtxMessage *message, unsigned char *ptr, unsigned char *dataEnd);
+static void PushOutputWord(DmtxMessage *msg, int value);
+static void PushOutputC40TextWord(DmtxMessage *msg, C40TextState *state, int value);
+static unsigned char *DecodeSchemeAsciiStd(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd);
+static unsigned char *DecodeSchemeAsciiExt(DmtxMessage *msg, unsigned char *ptr);
+static unsigned char *DecodeSchemeC40Text(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd, DmtxSchemeDecode encScheme);
+static unsigned char *DecodeSchemeX12(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd);
+static unsigned char *DecodeSchemeEdifact(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd);
+static unsigned char *DecodeSchemeBase256(DmtxMessage *msg, unsigned char *ptr, unsigned char *dataEnd);
 /* static unsigned char UnRandomize253State(unsigned char codewordValue, int codewordPosition); */
 static unsigned char UnRandomize255State(unsigned char value, int idx);
-static int PopulateArrayFromMatrix(DmtxMessage *message, DmtxImage *image, DmtxRegion *region);
-static void TallyModuleJumps(DmtxImage *image, DmtxRegion *region, int tally[][24], int xOrigin, int yOrigin, int mapWidth, int mapHeight, DmtxDirection dir);
-static int PopulateArrayFromMosaic(DmtxMessage *message, DmtxImage *image, DmtxRegion *region);
+static void TallyModuleJumps(DmtxDecode *dec, DmtxRegion *reg, int tally[][24], int xOrigin, int yOrigin, int mapWidth, int mapHeight, DmtxDirection dir);
+static DmtxPassFail PopulateArrayFromMatrix(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg);
 
 /* dmtxencode.c */
-static void AddPadChars(unsigned char *buf, int *bufSize, int paddedSize);
+static int AddPadChars(unsigned char *buf, int *bufSize, int paddedSize);
 static unsigned char Randomize253State(unsigned char codewordValue, int codewordPosition);
 static unsigned char Randomize255State(unsigned char codewordValue, int codewordPosition);
 static void PrintPattern(DmtxEncode *encode);
@@ -172,27 +183,30 @@ static void PatternShapeSpecial2(unsigned char *modules, int mappingRows, int ma
 static void PatternShapeSpecial3(unsigned char *modules, int mappingRows, int mappingCols, unsigned char *codeword, int moduleOnColor);
 static void PatternShapeSpecial4(unsigned char *modules, int mappingRows, int mappingCols, unsigned char *codeword, int moduleOnColor);
 static void PlaceModule(unsigned char *modules, int mappingRows, int mappingCols, int row, int col,
-      unsigned char *codeword, DmtxBitMask mask, int moduleOnColor);
+      unsigned char *codeword, int mask, int moduleOnColor);
 
 /* dmtxreedsol.c */
-static void GenReedSolEcc(DmtxMessage *message, int sizeidx);
+static void GenReedSolEcc(DmtxMessage *msg, int sizeidx);
 static int DecodeCheckErrors(unsigned char *code, int sizeIdx, int fix);
 static int GfProduct(int a, int b);
 static int GfDoublify(int a, int b);
 
 /* dmtxscangrid.c */
-static DmtxScanGrid InitScanGrid(DmtxImage *img, int smallestFeature);
-static DmtxPixelLoc IncrementPixelProgress(DmtxScanGrid *cross);
-static void SetDerivedFields(DmtxScanGrid *cross);
-static DmtxPixelLoc GetGridCoordinates(DmtxScanGrid *grid);
+static DmtxScanGrid InitScanGrid(DmtxDecode *dec);
+static int PopGridLocation(DmtxScanGrid *grid, /*@out@*/ DmtxPixelLoc *locPtr);
+static int GetGridCoordinates(DmtxScanGrid *grid, /*@out@*/ DmtxPixelLoc *locPtr);
+static void SetDerivedFields(DmtxScanGrid *grid);
 
 /* dmtxsymbol.c */
 static int FindCorrectSymbolSize(int dataWords, int symbolShape);
 
+/* dmtximage.c */
+static int GetBitsPerPixel(int pack);
+
 static const int dmtxNeighborNone = 8;
 static const int dmtxPatternX[] = { -1,  0,  1,  1,  1,  0, -1, -1 };
 static const int dmtxPatternY[] = { -1, -1, -1,  0,  1,  1,  1,  0 };
-static const DmtxPointFlow dmtxBlankEdge = { 0, 0, 0, -1, { -1, -1 } };
+static const DmtxPointFlow dmtxBlankEdge = { 0, 0, 0, DmtxUndefined, { -1, -1 } };
 
 /*@ +charint @*/
 
