@@ -1,9 +1,9 @@
 /*
 libdmtx - Data Matrix Encoding/Decoding Library
 
-Copyright (c) 2008 Mike Laughton
-Copyright (c) 2008 Ryan Raasch
-Copyright (c) 2008 Olivier Guilyardi
+Copyright (C) 2008, 2009 Mike Laughton
+Copyright (C) 2008 Ryan Raasch
+Copyright (C) 2008 Olivier Guilyardi
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
@@ -22,22 +22,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 Contact: mike@dragonflylogic.com
 */
 
-/* $Id: dmtxread.c 514 2008-11-19 17:10:44Z mblaughton $ */
+/* $Id: dmtxread.c 750 2009-02-25 20:15:53Z mblaughton $ */
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <getopt.h>
-#include <errno.h>
-#include <ctype.h>
-#include <math.h>
-#include <stdarg.h>
-#include <assert.h>
-#include <sysexits.h>
-#include <magick/api.h>
-#include <dmtx.h>
 #include "dmtxread.h"
-#include "../common/dmtxutil.h"
 
 char *programName;
 
@@ -50,200 +37,200 @@ char *programName;
 int
 main(int argc, char *argv[])
 {
+   char *filePath;
+   int i;
    int err;
-   int fileIndex;
-   int pageIndex;
-   int imgWidth, imgHeight;
-   int fileScanCount, globalScanCount;
-   int nFiles;
+   int fileIndex, imgPageIndex;
+   int fileCount;
+   int imgScanCount, pageScanCount;
+   int width, height;
+   unsigned char *pxl;
    UserOptions opt;
-   DmtxTime  msec, *timeout;
+   DmtxTime timeout;
    DmtxImage *img;
-   DmtxDecode dec;
-   DmtxRegion reg;
+   DmtxDecode *dec;
+   DmtxRegion *reg;
    DmtxMessage *msg;
-   ImageReader reader;
+   MagickBooleanType success;
+   MagickWand *wand;
 
-   SetOptionDefaults(&opt);
-   InitializeMagick(*argv);
+   opt = GetDefaultOptions();
 
    err = HandleArgs(&opt, &fileIndex, &argc, &argv);
-   if(err != DMTX_SUCCESS)
+   if(err != DmtxPass)
       ShowUsage(EX_USAGE);
 
-   timeout = (opt.timeoutMS == -1) ? NULL : &msec;
+   fileCount = (argc == fileIndex) ? 1 : argc - fileIndex;
 
-   reader.image = NULL;
-   nFiles = argc - fileIndex;
-   globalScanCount = 0;
+   MagickWandGenesis();
 
-   /* Loop once for each page of each image listed in parameters */
-   for(pageIndex = 0; fileIndex < argc || (nFiles == 0 && fileIndex == argc);) {
+   /* Loop once for each image named on command line */
+   imgScanCount = 0;
+   for(i = 0; i < fileCount; i++) {
 
-      /* Reset timeout for each new image */
-      if(timeout != NULL)
-         msec = dmtxTimeAdd(dmtxTimeNow(), opt.timeoutMS);
+      /* Open image from file or stream (might contain multiple pages) */
+      filePath = (argc == fileIndex) ? "-" : argv[fileIndex++];
 
-      /* Open image file/stream */
-      if(!reader.image) {
-         if(argc == fileIndex)
-            OpenImage(&reader, "-", opt.resolution);
-         else
-            OpenImage(&reader, argv[fileIndex], opt.resolution);
+      wand = NewMagickWand();
+      if(wand == NULL) {
+         FatalError(EX_OSERR, "Magick error");
+      }
 
-         if(!reader.image) {
-            fileIndex++;
-            continue;
+      if(opt.dpi != DmtxUndefined) {
+         success = MagickSetResolution(wand, (double)opt.dpi, (double)opt.dpi);
+         if(success == MagickFalse) {
+            CleanupMagick(&wand, DmtxTrue);
+            FatalError(EX_OSERR, "Magick error");
          }
       }
 
-      /* Read next image page (many formats are single-page only) */
-      img = ReadImagePage(&reader, pageIndex++);
-
-      /* If requested page did not load then move to the next image */
-      if(img == NULL) {
-         CloseImage(&reader);
-         fileIndex++;
-         pageIndex = 0;
-         continue;
+      success = MagickReadImage(wand, filePath);
+      if(success == MagickFalse) {
+         CleanupMagick(&wand, DmtxTrue);
+         FatalError(EX_OSERR, "Magick error");
       }
 
-      imgWidth = dmtxImageGetProp(img, DmtxPropWidth);
-      imgHeight = dmtxImageGetProp(img, DmtxPropHeight);
+      width = MagickGetImageWidth(wand);
+      height = MagickGetImageHeight(wand);
 
-      assert(img->pageCount > 0 && pageIndex <= img->pageCount);
+      /* Loop once for each page within image */
+      MagickResetIterator(wand);
+      for(imgPageIndex = 0; MagickNextImage(wand) != MagickFalse; imgPageIndex++) {
 
-      /* Initialize decode struct for newly loaded image */
-      dec = dmtxDecodeStructInit(img);
-
-      err = dmtxDecodeSetProp(&dec, DmtxPropScanGap, opt.scanGap);
-      assert(err == DMTX_SUCCESS);
-
-      if(opt.edgeMin != -1) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropEdgeMin, opt.edgeMin);
-         assert(err == DMTX_SUCCESS);
-      }
-
-      if(opt.edgeMax != -1) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropEdgeMax, opt.edgeMax);
-         assert(err == DMTX_SUCCESS);
-      }
-
-      if(opt.squareDevn != -1) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropSquareDevn, opt.squareDevn);
-         assert(err == DMTX_SUCCESS);
-      }
-
-      err = dmtxDecodeSetProp(&dec, DmtxPropSymbolSize, opt.sizeIdxExpected);
-      assert(err == DMTX_SUCCESS);
-
-      err = dmtxDecodeSetProp(&dec, DmtxPropEdgeThresh, opt.edgeThresh);
-      assert(err == DMTX_SUCCESS);
-
-      if(opt.xMin) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropXmin, ScaleNumberString(opt.xMin, imgWidth));
-         assert(err == DMTX_SUCCESS);
-      }
-
-      if(opt.xMax) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropXmax, ScaleNumberString(opt.xMax, imgWidth));
-         assert(err == DMTX_SUCCESS);
-      }
-
-      if(opt.yMin) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropYmin, ScaleNumberString(opt.yMin, imgHeight));
-         assert(err == DMTX_SUCCESS);
-      }
-
-      if(opt.yMax) {
-         err = dmtxDecodeSetProp(&dec, DmtxPropYmax, ScaleNumberString(opt.yMax, imgHeight));
-         assert(err == DMTX_SUCCESS);
-      }
-
-      err = dmtxDecodeSetProp(&dec, DmtxPropShrinkMin, opt.shrinkMin);
-      assert(err == DMTX_SUCCESS);
-
-      err = dmtxDecodeSetProp(&dec, DmtxPropShrinkMax, opt.shrinkMax);
-      assert(err == DMTX_SUCCESS);
-
-      /* Loop once for each detected barcode region */
-      for(fileScanCount = 0;;) {
-
-         /* Find next barcode region within image, but do not decode yet */
-         reg = dmtxDecodeFindNextRegion(&dec, timeout);
-
-         /* Finished file or ran out of time before finding another region */
-         if(reg.found != DMTX_REGION_FOUND)
-            break;
-
-         /* Decode region based on requested barcode mode */
-         if(opt.mosaic)
-            msg = dmtxDecodeMosaicRegion(img, &reg, opt.correctionsMax);
-         else
-            msg = dmtxDecodeMatrixRegion(img, &reg, opt.correctionsMax);
-
-         if(msg == NULL)
+         /* If requested, only scan specific page */
+         if(opt.page != DmtxUndefined && opt.page - 1 != imgPageIndex)
             continue;
 
-         PrintDecodedOutput(&opt, img, &reg, msg, pageIndex);
-         fileScanCount++;
+         /* Reset timeout for each new page */
+         if(opt.timeoutMS != DmtxUndefined)
+            timeout = dmtxTimeAdd(dmtxTimeNow(), opt.timeoutMS);
 
-         dmtxMessageFree(&msg);
+         /* Allocate memory for pixel data */
+         pxl = (unsigned char *)malloc(3 * width * height * sizeof(unsigned char));
+         if(pxl == NULL) {
+            CleanupMagick(&wand, DmtxFalse);
+            FatalError(EX_OSERR, "malloc() error");
+         }
 
-         if(opt.stopAfter != -1 && fileScanCount >= opt.stopAfter)
-            break;
+         /* Copy pixels to known format */
+         success = MagickGetImagePixels(wand, 0, 0, width, height, "RGB", CharPixel, pxl);
+         if(success == MagickFalse || pxl == NULL) {
+            CleanupMagick(&wand, DmtxTrue);
+            FatalError(EX_OSERR, "malloc() error");
+         }
+
+         /* Initialize libdmtx image */
+         img = dmtxImageCreate(pxl, width, height, DmtxPack24bppRGB);
+         if(img == NULL) {
+            CleanupMagick(&wand, DmtxFalse);
+            FatalError(EX_SOFTWARE, "dmtxImageCreate() error");
+         }
+
+         dmtxImageSetProp(img, DmtxPropImageFlip, DmtxFlipNone);
+
+         /* Initialize scan */
+         dec = dmtxDecodeCreate(img, opt.shrinkMin);
+         if(dec == NULL) {
+            CleanupMagick(&wand, DmtxFalse);
+            FatalError(EX_SOFTWARE, "decode create error");
+         }
+
+         err = SetDecodeOptions(dec, img, &opt);
+         if(err != DmtxPass) {
+            CleanupMagick(&wand, DmtxFalse);
+            FatalError(EX_SOFTWARE, "decode option error");
+         }
+
+         /* Find and decode every barcode on page */
+         pageScanCount = 0;
+         for(;;) {
+            /* Find next barcode region within image, but do not decode yet */
+            if(opt.timeoutMS == DmtxUndefined)
+               reg = dmtxRegionFindNext(dec, NULL);
+            else
+               reg = dmtxRegionFindNext(dec, &timeout);
+
+            /* Finished file or ran out of time before finding another region */
+            if(reg == NULL)
+               break;
+
+            /* Decode region based on requested barcode mode */
+            if(opt.mosaic == DmtxTrue)
+               msg = dmtxDecodeMosaicRegion(dec, reg, opt.correctionsMax);
+            else
+               msg = dmtxDecodeMatrixRegion(dec, reg, opt.correctionsMax);
+
+            if(msg != NULL) {
+               PrintStats(dec, reg, msg, imgPageIndex, &opt);
+               PrintMessage(reg, msg, &opt);
+
+               pageScanCount++;
+               imgScanCount++;
+
+               dmtxMessageDestroy(&msg);
+            }
+
+            dmtxRegionDestroy(&reg);
+
+            if(opt.stopAfter != DmtxUndefined && imgScanCount >= opt.stopAfter)
+               break;
+         }
+
+         if(opt.diagnose == DmtxTrue)
+            WriteDiagnosticImage(dec, "debug.pnm");
+
+         dmtxDecodeDestroy(&dec);
+         dmtxImageDestroy(&img);
+         free(pxl);
       }
-      globalScanCount += fileScanCount;
 
-      if(opt.diagnose)
-         WriteDiagnosticImage(&dec, &reg, "debug.pnm");
-
-      dmtxDecodeStructDeInit(&dec);
-      dmtxImageFree(&img);
+      CleanupMagick(&wand, DmtxFalse);
    }
 
-   DestroyMagick();
+   MagickWandTerminus();
 
-   exit((globalScanCount > 0) ? EX_OK : 1);
+   exit((imgScanCount > 0) ? EX_OK : 1);
 }
 
 /**
  *
  *
  */
-static void
-SetOptionDefaults(UserOptions *opt)
+static UserOptions
+GetDefaultOptions(void)
 {
-   UserOptions option;
+   UserOptions opt;
 
-   memset(&option, 0x00, sizeof(UserOptions));
+   memset(&opt, 0x00, sizeof(UserOptions));
 
-   /* Set default options */
-   option.codewords = 0;
-   option.edgeMin = -1;
-   option.edgeMax = -1;
-   option.squareDevn = -1;
-   option.scanGap = 2;
-   option.timeoutMS = -1;
-   option.newline = 0;
-   option.resolution = NULL;
-   option.sizeIdxExpected = DmtxSymbolShapeAuto;
-   option.edgeThresh = 5;
-   option.xMin = NULL;
-   option.xMax = NULL;
-   option.yMin = NULL;
-   option.yMax = NULL;
-   option.correctionsMax = -1;
-   option.diagnose = 0;
-   option.mosaic = 0;
-   option.stopAfter = -1;
-   option.pageNumber = 0;
-   option.corners = 0;
-   option.shrinkMin = 1;
-   option.shrinkMax = 1;
-   option.verbose = 0;
+   /* Default options */
+   opt.codewords = DmtxFalse;
+   opt.edgeMin = DmtxUndefined;
+   opt.edgeMax = DmtxUndefined;
+   opt.scanGap = 2;
+   opt.timeoutMS = DmtxUndefined;
+   opt.newline = DmtxFalse;
+   opt.page = DmtxUndefined;
+   opt.squareDevn = DmtxUndefined;
+   opt.dpi = DmtxUndefined;
+   opt.sizeIdxExpected = DmtxSymbolShapeAuto;
+   opt.edgeThresh = 5;
+   opt.xMin = NULL;
+   opt.xMax = NULL;
+   opt.yMin = NULL;
+   opt.yMax = NULL;
+   opt.correctionsMax = DmtxUndefined;
+   opt.diagnose = DmtxFalse;
+   opt.mosaic = DmtxFalse;
+   opt.stopAfter = DmtxUndefined;
+   opt.pageNumbers = DmtxFalse;
+   opt.corners = DmtxFalse;
+   opt.shrinkMin = 1;
+   opt.shrinkMax = 1;
+   opt.unicode = DmtxFalse;
+   opt.verbose = DmtxFalse;
 
-   *opt = option;
+   return opt;
 }
 
 /**
@@ -252,9 +239,9 @@ SetOptionDefaults(UserOptions *opt)
  * @param  argcp pointer to argument count
  * @param  argvp pointer to argument list
  * @param  fileIndex pointer to index of first non-option arg (if successful)
- * @return DMTX_SUCCESS | DMTX_FAILURE
+ * @return DmtxPass | DmtxFail
  */
-static int
+static DmtxPassFail
 HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
 {
    int i;
@@ -271,6 +258,7 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
          {"list-formats",     no_argument,       NULL, 'l'},
          {"milliseconds",     required_argument, NULL, 'm'},
          {"newline",          no_argument,       NULL, 'n'},
+         {"page",             required_argument, NULL, 'p'},
          {"square-deviation", required_argument, NULL, 'q'},
          {"resolution",       required_argument, NULL, 'r'},
          {"symbol-size",      required_argument, NULL, 's'},
@@ -283,9 +271,10 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
          {"diagnose",         no_argument,       NULL, 'D'},
          {"mosaic",           no_argument,       NULL, 'M'},
          {"stop-after",       required_argument, NULL, 'N'},
-         {"page-number",      no_argument,       NULL, 'P'},
+         {"page-numbers",     no_argument,       NULL, 'P'},
          {"corners",          no_argument,       NULL, 'R'},
          {"shrink",           required_argument, NULL, 'S'},
+         {"unicode",          no_argument,       NULL, 'U'},
          {"verbose",          no_argument,       NULL, 'v'},
          {"version",          no_argument,       NULL, 'V'},
          {"help",             no_argument,       NULL,  0 },
@@ -297,7 +286,8 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
    *fileIndex = 0;
 
    for(;;) {
-      optchr = getopt_long(*argcp, *argvp, "ce:E:g:lm:nq:r:s:t:x:X:y:Y:vC:DMN:PRS:V", longOptions, &longIndex);
+      optchr = getopt_long(*argcp, *argvp,
+            "ce:E:g:lm:np:q:r:s:t:x:X:y:Y:vC:DMN:PRS:UV", longOptions, &longIndex);
       if(optchr == -1)
          break;
 
@@ -305,44 +295,50 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
          case 0: /* --help */
             ShowUsage(EX_OK);
             break;
-         case 'l': /* --help */
+         case 'l':
             ListImageFormats();
             exit(EX_OK);
             break;
          case 'c':
-            opt->codewords = 1;
+            opt->codewords = DmtxTrue;
             break;
          case 'e':
             err = StringToInt(&(opt->edgeMin), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->edgeMin <= 0 || *ptr != '\0')
+            if(err != DmtxPass || opt->edgeMin <= 0 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid edge length specified \"%s\""), optarg);
             break;
          case 'E':
             err = StringToInt(&(opt->edgeMax), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->edgeMax <= 0 || *ptr != '\0')
+            if(err != DmtxPass || opt->edgeMax <= 0 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid edge length specified \"%s\""), optarg);
             break;
          case 'g':
             err = StringToInt(&(opt->scanGap), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->scanGap <= 0 || *ptr != '\0')
+            if(err != DmtxPass || opt->scanGap <= 0 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid gap specified \"%s\""), optarg);
             break;
          case 'm':
             err = StringToInt(&(opt->timeoutMS), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->timeoutMS < 0 || *ptr != '\0')
+            if(err != DmtxPass || opt->timeoutMS < 0 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid timeout (in milliseconds) specified \"%s\""), optarg);
             break;
          case 'n':
-            opt->newline = 1;
+            opt->newline = DmtxTrue;
             break;
+         case 'p':
+            err = StringToInt(&(opt->page), optarg, &ptr);
+            if(err != DmtxPass || opt->page < 1 || *ptr != '\0')
+               FatalError(EX_USAGE, _("Invalid page specified \"%s\""), optarg);
          case 'q':
             err = StringToInt(&(opt->squareDevn), optarg, &ptr);
-            if(err != DMTX_SUCCESS || *ptr != '\0' ||
+            if(err != DmtxPass || *ptr != '\0' ||
                   opt->squareDevn < 0 || opt->squareDevn > 90)
                FatalError(EX_USAGE, _("Invalid squareness deviation specified \"%s\""), optarg);
             break;
          case 'r':
-            opt->resolution = optarg;
+            err = StringToInt(&(opt->dpi), optarg, &ptr);
+            if(err != DmtxPass || *ptr != '\0' || opt->dpi < 1)
+               FatalError(EX_USAGE, _("Invalid resolution specified \"%s\""), optarg);
             break;
          case 's':
             /* Determine correct barcode size and/or shape */
@@ -356,19 +352,19 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
                opt->sizeIdxExpected = DmtxSymbolRectAuto;
             }
             else {
-               for(i = 0; i < DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT; i++) {
+               for(i = 0; i < DmtxSymbolSquareCount + DmtxSymbolRectCount; i++) {
                   if(strncmp(optarg, symbolSizes[i], 8) == 0) {
                      opt->sizeIdxExpected = i;
                      break;
                   }
                }
-               if(i == DMTX_SYMBOL_SQUARE_COUNT + DMTX_SYMBOL_RECT_COUNT)
-                  return DMTX_FAILURE;
+               if(i == DmtxSymbolSquareCount + DmtxSymbolRectCount)
+                  return DmtxFail;
             }
             break;
          case 't':
             err = StringToInt(&(opt->edgeThresh), optarg, &ptr);
-            if(err != DMTX_SUCCESS || *ptr != '\0' ||
+            if(err != DmtxPass || *ptr != '\0' ||
                   opt->edgeThresh < 1 || opt->edgeThresh > 100)
                FatalError(EX_USAGE, _("Invalid edge threshold specified \"%s\""), optarg);
             break;
@@ -385,49 +381,54 @@ HandleArgs(UserOptions *opt, int *fileIndex, int *argcp, char **argvp[])
             opt->yMax = optarg;
             break;
          case 'v':
-            opt->verbose = 1;
+            opt->verbose = DmtxTrue;
             break;
          case 'C':
             err = StringToInt(&(opt->correctionsMax), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->correctionsMax < 0 || *ptr != '\0')
+            if(err != DmtxPass || opt->correctionsMax < 0 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid max corrections specified \"%s\""), optarg);
             break;
          case 'D':
-            opt->diagnose = 1;
+            opt->diagnose = DmtxTrue;
             break;
          case 'M':
-            opt->mosaic = 1;
+            opt->mosaic = DmtxTrue;
             break;
          case 'N':
             err = StringToInt(&(opt->stopAfter), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->stopAfter < 1 || *ptr != '\0')
+            if(err != DmtxPass || opt->stopAfter < 1 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid count specified \"%s\""), optarg);
             break;
          case 'P':
-            opt->pageNumber = 1;
+            opt->pageNumbers = DmtxTrue;
             break;
          case 'R':
-            opt->corners = 1;
+            opt->corners = DmtxTrue;
             break;
          case 'S':
-            err = StringToInt(&(opt->shrinkMax), optarg, &ptr);
-            if(err != DMTX_SUCCESS || opt->shrinkMax < 1 || *ptr != '\0')
+            err = StringToInt(&(opt->shrinkMin), optarg, &ptr);
+            if(err != DmtxPass || opt->shrinkMin < 1 || *ptr != '\0')
                FatalError(EX_USAGE, _("Invalid shrink factor specified \"%s\""), optarg);
-            /* XXX later also popular shrinkMin based on N-N range */
+
+            /* XXX later populate shrinkMax based on specified N-N range */
+            opt->shrinkMax = opt->shrinkMin;
+            break;
+         case 'U':
+            opt->unicode = DmtxTrue;
             break;
          case 'V':
-            fprintf(stdout, "%s version %s\n", programName, DMTX_VERSION);
+            fprintf(stdout, "%s version %s\n", programName, DmtxVersion);
             fprintf(stdout, "libdmtx version %s\n", dmtxVersion());
             exit(EX_OK);
             break;
          default:
-            return DMTX_FAILURE;
+            return DmtxFail;
             break;
       }
    }
    *fileIndex = optind;
 
-   return DMTX_SUCCESS;
+   return DmtxPass;
 }
 
 /**
@@ -461,6 +462,7 @@ OPTIONS:\n"), programName, programName);
   -l, --list-formats          list supported image formats\n\
   -m, --milliseconds=N        stop scan after N milliseconds (per image)\n\
   -n, --newline               print newline character at the end of decoded data\n\
+  -p, --page=N                only scan Nth page of images\n\
   -q, --square-deviation=N    allow non-squareness of corners in degrees (0-90)\n"));
       fprintf(stdout, _("\
   -r, --resolution=N          resolution for vector images (PDF, SVG, etc...)\n\
@@ -480,9 +482,10 @@ OPTIONS:\n"), programName, programName);
   -M, --mosaic                interpret detected regions as Data Mosaic barcodes\n"));
       fprintf(stdout, _("\
   -N, --stop-after=N          stop scanning after Nth barcode is returned\n\
-  -P, --page-number           prefix decoded message with fax/tiff page number\n\
+  -P, --page-numbers          prefix decoded message with fax/tiff page number\n\
   -R, --corners               prefix decoded message with corner locations\n\
   -S, --shrink=N              internally shrink image by a factor of N\n\
+  -U, --unicode               print Extended ASCII in Unicode (UTF-8)\n\
   -v, --verbose               use verbose messages\n\
   -V, --version               print program version information\n\
       --help                  display this help and exit\n"));
@@ -492,6 +495,274 @@ OPTIONS:\n"), programName, programName);
    exit(status);
 }
 
+/**
+ *
+ *
+ */
+static DmtxPassFail
+SetDecodeOptions(DmtxDecode *dec, DmtxImage *img, UserOptions *opt)
+{
+   int err;
+
+#define RETURN_IF_FAILED(e) if(e != DmtxPass) { return DmtxFail; }
+
+   err = dmtxDecodeSetProp(dec, DmtxPropScanGap, opt->scanGap);
+   RETURN_IF_FAILED(err)
+
+   if(opt->edgeMin != DmtxUndefined) {
+      err = dmtxDecodeSetProp(dec, DmtxPropEdgeMin, opt->edgeMin);
+      RETURN_IF_FAILED(err)
+   }
+
+   if(opt->edgeMax != DmtxUndefined) {
+      err = dmtxDecodeSetProp(dec, DmtxPropEdgeMax, opt->edgeMax);
+      RETURN_IF_FAILED(err)
+   }
+
+   if(opt->squareDevn != DmtxUndefined) {
+      err = dmtxDecodeSetProp(dec, DmtxPropSquareDevn, opt->squareDevn);
+      RETURN_IF_FAILED(err)
+   }
+
+   err = dmtxDecodeSetProp(dec, DmtxPropSymbolSize, opt->sizeIdxExpected);
+   RETURN_IF_FAILED(err)
+
+   err = dmtxDecodeSetProp(dec, DmtxPropEdgeThresh, opt->edgeThresh);
+   RETURN_IF_FAILED(err)
+
+   if(opt->xMin) {
+      err = dmtxDecodeSetProp(dec, DmtxPropXmin, ScaleNumberString(opt->xMin, img->width));
+      RETURN_IF_FAILED(err)
+   }
+
+   if(opt->xMax) {
+      err = dmtxDecodeSetProp(dec, DmtxPropXmax, ScaleNumberString(opt->xMax, img->width));
+      RETURN_IF_FAILED(err)
+   }
+
+   if(opt->yMin) {
+      err = dmtxDecodeSetProp(dec, DmtxPropYmin, ScaleNumberString(opt->yMin, img->height));
+      RETURN_IF_FAILED(err)
+   }
+
+   if(opt->yMax) {
+      err = dmtxDecodeSetProp(dec, DmtxPropYmax, ScaleNumberString(opt->yMax, img->height));
+      RETURN_IF_FAILED(err)
+   }
+
+#undef RETURN_IF_FAILED
+
+   return DmtxPass;
+}
+
+/**
+ * @brief  Print decoded message to standard output
+ * @param  opt runtime options from defaults or command line
+ * @param  dec pointer to DmtxDecode struct
+ * @return DmtxPass | DmtxFail
+ */
+static DmtxPassFail
+PrintStats(DmtxDecode *dec, DmtxRegion *reg, DmtxMessage *msg,
+      int imgPageIndex, UserOptions *opt)
+{
+   int height;
+   int dataWordLength;
+   int rotateInt;
+   double rotate;
+   DmtxVector2 p00, p10, p11, p01;
+
+   height = dmtxDecodeGetProp(dec, DmtxPropHeight);
+
+   p00.X = p00.Y = p10.Y = p01.X = 0.0;
+   p10.X = p01.Y = p11.X = p11.Y = 1.0;
+   dmtxMatrix3VMultiplyBy(&p00, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&p10, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&p11, reg->fit2raw);
+   dmtxMatrix3VMultiplyBy(&p01, reg->fit2raw);
+
+   dataWordLength = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, reg->sizeIdx);
+   if(opt->verbose == DmtxTrue) {
+
+      rotate = (2 * M_PI) + (atan2(reg->fit2raw[0][1], reg->fit2raw[1][1]) -
+            atan2(reg->fit2raw[1][0], reg->fit2raw[0][0])) / 2.0;
+
+      rotateInt = (int)(rotate * 180/M_PI + 0.5);
+      if(rotateInt >= 360)
+         rotateInt -= 360;
+
+      fprintf(stdout, "--------------------------------------------------\n");
+      fprintf(stdout, "       Matrix Size: %d x %d\n",
+            dmtxGetSymbolAttribute(DmtxSymAttribSymbolRows, reg->sizeIdx),
+            dmtxGetSymbolAttribute(DmtxSymAttribSymbolCols, reg->sizeIdx));
+      fprintf(stdout, "    Data Codewords: %d (capacity %d)\n",
+            dataWordLength - msg->padCount, dataWordLength);
+      fprintf(stdout, "   Error Codewords: %d\n",
+            dmtxGetSymbolAttribute(DmtxSymAttribSymbolErrorWords, reg->sizeIdx));
+      fprintf(stdout, "      Data Regions: %d x %d\n",
+            dmtxGetSymbolAttribute(DmtxSymAttribHorizDataRegions, reg->sizeIdx),
+            dmtxGetSymbolAttribute(DmtxSymAttribVertDataRegions, reg->sizeIdx));
+      fprintf(stdout, "Interleaved Blocks: %d\n",
+            dmtxGetSymbolAttribute(DmtxSymAttribInterleavedBlocks, reg->sizeIdx));
+      fprintf(stdout, "    Rotation Angle: %d\n", rotateInt);
+      fprintf(stdout, "          Corner 0: (%0.1f, %0.1f)\n", p00.X, height - 1 - p00.Y);
+      fprintf(stdout, "          Corner 1: (%0.1f, %0.1f)\n", p10.X, height - 1 - p10.Y);
+      fprintf(stdout, "          Corner 2: (%0.1f, %0.1f)\n", p11.X, height - 1 - p11.Y);
+      fprintf(stdout, "          Corner 3: (%0.1f, %0.1f)\n", p01.X, height - 1 - p01.Y);
+      fprintf(stdout, "--------------------------------------------------\n");
+   }
+
+   if(opt->pageNumbers == DmtxTrue)
+      fprintf(stdout, "%d:", imgPageIndex + 1);
+
+   if(opt->corners == DmtxTrue) {
+      fprintf(stdout, "%d,%d:", (int)(p00.X + 0.5), height - 1 - (int)(p00.Y + 0.5));
+      fprintf(stdout, "%d,%d:", (int)(p10.X + 0.5), height - 1 - (int)(p10.Y + 0.5));
+      fprintf(stdout, "%d,%d:", (int)(p11.X + 0.5), height - 1 - (int)(p11.Y + 0.5));
+      fprintf(stdout, "%d,%d:", (int)(p01.X + 0.5), height - 1 - (int)(p01.Y + 0.5));
+   }
+
+   return DmtxPass;
+}
+
+/**
+ *
+ *
+ */
+static DmtxPassFail
+PrintMessage(DmtxRegion *reg, DmtxMessage *msg, UserOptions *opt)
+{
+   int i;
+   int remainingDataWords;
+   int dataWordLength;
+
+   if(opt->codewords == DmtxTrue) {
+      dataWordLength = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, reg->sizeIdx);
+      for(i = 0; i < msg->codeSize; i++) {
+         remainingDataWords = dataWordLength - i;
+         if(remainingDataWords > msg->padCount)
+            fprintf(stdout, "%c:%03d\n", 'd', msg->code[i]);
+         else if(remainingDataWords > 0)
+            fprintf(stdout, "%c:%03d\n", 'p', msg->code[i]);
+         else
+            fprintf(stdout, "%c:%03d\n", 'e', msg->code[i]);
+      }
+   }
+   else {
+      if(opt->unicode == DmtxTrue) {
+         for(i = 0; i < msg->outputIdx; i++) {
+            if(msg->output[i] < 128) {
+               fputc(msg->output[i], stdout);
+            }
+            else if(msg->output[i] < 192) {
+              fputc(0xc2, stdout);
+              fputc(msg->output[i], stdout);
+            }
+            else {
+               fputc(0xc3, stdout);
+               fputc(msg->output[i] - 64, stdout);
+            }
+         }
+      }
+      else {
+         fwrite(msg->output, sizeof(char), msg->outputIdx, stdout);
+      }
+
+      if(opt->newline)
+         fputc('\n', stdout);
+   }
+
+   return DmtxPass;
+}
+
+/**
+ *
+ *
+ */
+static void
+CleanupMagick(MagickWand **wand, int magickError)
+{
+   char *excMessage;
+   ExceptionType excSeverity;
+
+   if(magickError == DmtxTrue) {
+      excMessage = MagickGetException(*wand, &excSeverity);
+      fprintf(stderr, "%s %s %lu %s\n", GetMagickModule(), excMessage);
+      MagickRelinquishMemory(excMessage);
+   }
+
+   if(*wand != NULL) {
+      DestroyMagickWand(*wand);
+      *wand = NULL;
+   }
+}
+
+/**
+ * @brief  List supported input image formats on stdout
+ * @return void
+ */
+static void
+ListImageFormats(void)
+{
+   int i, index;
+   int row, rowCount;
+   int col, colCount;
+   unsigned long totalCount;
+   char **list;
+
+   list = MagickQueryFormats("*", &totalCount);
+
+   if(list == NULL)
+      return;
+
+   fprintf(stdout, "\n");
+
+   colCount = 7;
+   rowCount = totalCount/colCount;
+   if(totalCount % colCount)
+      rowCount++;
+
+   for(i = 0; i < colCount * rowCount; i++) {
+      col = i%colCount;
+      row = i/colCount;
+      index = col*rowCount + row;
+      fprintf(stdout, "%10s", (index < totalCount) ? list[col*rowCount+row] : " ");
+      fprintf(stdout, "%s", (col+1 < colCount) ? " " : "\n");
+   }
+   fprintf(stdout, "\n");
+
+   MagickRelinquishMemory(list);
+}
+
+/**
+ *
+ *
+ */
+static void
+WriteDiagnosticImage(DmtxDecode *dec, char *imagePath)
+{
+   int totalBytes, headerBytes;
+   int bytesWritten;
+   unsigned char *pnm;
+   FILE *fp;
+
+   fp = fopen(imagePath, "wb");
+   if(fp == NULL) {
+      perror(programName);
+      FatalError(EX_CANTCREAT, _("Unable to create image \"%s\""), imagePath);
+   }
+
+   pnm = dmtxDecodeCreateDiagnostic(dec, &totalBytes, &headerBytes, 0);
+   if(pnm == NULL)
+      FatalError(EX_OSERR, _("Unable to create diagnostic image"));
+
+   bytesWritten = fwrite(pnm, sizeof(unsigned char), totalBytes, fp);
+   if(bytesWritten != totalBytes)
+      FatalError(EX_IOERR, _("Unable to write diagnostic image"));
+
+   free(pnm);
+   fclose(fp);
+}
+
 /**
  *
  *
@@ -507,7 +778,7 @@ ScaleNumberString(char *s, int extent)
    assert(s != NULL);
 
    err = StringToInt(&numValue, s, &terminate);
-   if(err != DMTX_SUCCESS)
+   if(err != DmtxPass)
       FatalError(EX_USAGE, _("Integer value required"));
 
    scaledValue = (*terminate == '%') ? (int)(0.01 * numValue * extent + 0.5) : numValue;
@@ -520,271 +791,3 @@ ScaleNumberString(char *s, int extent)
 
    return scaledValue;
 }
-
-/**
- * @brief  List supported input image formats on stdout
- * @return void
- */
-static void
-ListImageFormats(void)
-{
-   int i;
-   MagickInfo **formats;
-   ExceptionInfo exception;
-
-   GetExceptionInfo(&exception);
-   formats=GetMagickInfoArray(&exception);
-   CatchException(&exception);
-
-   if(formats) {
-      fprintf(stdout, "   Format  Description\n");
-      fprintf(stdout, "---------  ------------------------------------------------------\n");
-      for(i=0; formats[i] != 0; i++) {
-         if(formats[i]->stealth || !formats[i]->decoder)
-            continue;
-
-         fprintf(stdout, "%9s", formats[i]->name ? formats[i]->name : "");
-         if(formats[i]->description != (char *)NULL)
-            fprintf(stdout, "  %s\n",formats[i]->description);
-      }
-      free(formats);
-   }
-
-   DestroyExceptionInfo(&exception);
-}
-
-/**
- * @brief  Open an image input file
- * @param  reader Pointer to ImageReader struct
- * @param  imagePath Image path or "-" for stdin
- * @return DMTX_SUCCESS | DMTX_FAILURE
- */
-static int
-OpenImage(ImageReader *reader, char *imagePath, char *resolution)
-{
-   reader->image = NULL;
-   GetExceptionInfo(&reader->exception);
-   reader->info = CloneImageInfo((ImageInfo *) NULL);
-
-   strcpy((reader->info)->filename, imagePath);
-   if(resolution) {
-      reader->info->density = strdup(resolution);
-      reader->info->units = PixelsPerInchResolution;
-   }
-   reader->image = ReadImage(reader->info, &reader->exception);
-
-   switch(reader->exception.severity) {
-      case UndefinedException:
-         break;
-      case OptionError:
-      case MissingDelegateError:
-         fprintf(stderr,
-               "%s: unsupported file format. Use -l to for a list of available formats.\n",
-               imagePath);
-         break;
-      default:
-         CatchException(&reader->exception);
-         break;
-   }
-
-   if(!reader->image)
-      CloseImage(reader);
-
-   return (reader->image) ? DMTX_SUCCESS : DMTX_FAILURE;
-}
-
-/**
- * @brief  Read an image page
- * @param  reader pointer to ImageReader struct
- * @param  index page index
- * @return pointer to allocated DmtxImage or NULL
- */
-static DmtxImage *
-ReadImagePage(ImageReader *reader, int index)
-{
-   unsigned int dispatchResult;
-   int pageCount;
-   DmtxImage *image;
-
-   image = NULL;
-   if(reader->image) {
-      pageCount = GetImageListLength(reader->image);
-      if(index < pageCount) {
-         reader->image = GetImageFromList(reader->image, index);
-
-         image = dmtxImageMalloc(reader->image->columns, reader->image->rows);
-
-         if(image) {
-            image->pageCount = pageCount;
-            dispatchResult = DispatchImage(reader->image, 0, 0, reader->image->columns,
-                                           reader->image->rows, "RGB", CharPixel,
-                                           image->pxl, &reader->exception);
-            if(dispatchResult == MagickFail) {
-               dmtxImageFree(&image);
-               CatchException(&reader->exception);
-            }
-         }
-         else {
-            perror(programName);
-         }
-      }
-   }
-
-   return image;
-}
-
-/**
- * @brief  Close an image
- * @param  reader  pointer to ImageReader struct
- * @return void
- */
-static void
-CloseImage(ImageReader * reader)
-{
-   if(reader->image) {
-      DestroyImage(reader->image);
-      reader->image = NULL;
-   }
-
-   DestroyImageInfo(reader->info);
-   DestroyExceptionInfo(&reader->exception);
-}
-
-/**
- * @brief  Print decoded message to standard output
- * @param  opt runtime options from defaults or command line
- * @param  dec pointer to DmtxDecode struct
- * @return DMTX_SUCCESS | DMTX_FAILURE
- */
-static int
-PrintDecodedOutput(UserOptions *opt, DmtxImage *image,
-      DmtxRegion *reg, DmtxMessage *msg, int pageIndex)
-{
-   int i;
-   int height;
-   int dataWordLength;
-   int rotateInt;
-   double rotate;
-   DmtxVector2 p00, p10, p11, p01;
-
-   height = dmtxImageGetProp(image, DmtxPropScaledHeight);
-
-   p00.X = p00.Y = p10.Y = p01.X = 0.0;
-   p10.X = p01.Y = p11.X = p11.Y = 1.0;
-   dmtxMatrix3VMultiplyBy(&p00, reg->fit2raw);
-   dmtxMatrix3VMultiplyBy(&p10, reg->fit2raw);
-   dmtxMatrix3VMultiplyBy(&p11, reg->fit2raw);
-   dmtxMatrix3VMultiplyBy(&p01, reg->fit2raw);
-
-   dataWordLength = dmtxGetSymbolAttribute(DmtxSymAttribSymbolDataWords, reg->sizeIdx);
-   if(opt->verbose) {
-
-      rotate = (2 * M_PI) + (atan2(reg->fit2raw[0][1], reg->fit2raw[1][1]) -
-            atan2(reg->fit2raw[1][0], reg->fit2raw[0][0])) / 2.0;
-
-      rotateInt = (int)(rotate * 180/M_PI + 0.5);
-      if(rotateInt >= 360)
-         rotateInt -= 360;
-
-      fprintf(stdout, "--------------------------------------------------\n");
-      fprintf(stdout, "       Matrix Size: %d x %d\n",
-            dmtxGetSymbolAttribute(DmtxSymAttribSymbolRows, reg->sizeIdx),
-            dmtxGetSymbolAttribute(DmtxSymAttribSymbolCols, reg->sizeIdx));
-      fprintf(stdout, "    Data Codewords: %d (capacity %d)\n",
-            msg->outputIdx, dataWordLength);
-      fprintf(stdout, "   Error Codewords: %d\n",
-            dmtxGetSymbolAttribute(DmtxSymAttribSymbolErrorWords, reg->sizeIdx));
-      fprintf(stdout, "      Data Regions: %d x %d\n",
-            dmtxGetSymbolAttribute(DmtxSymAttribHorizDataRegions, reg->sizeIdx),
-            dmtxGetSymbolAttribute(DmtxSymAttribVertDataRegions, reg->sizeIdx));
-      fprintf(stdout, "Interleaved Blocks: %d\n",
-            dmtxGetSymbolAttribute(DmtxSymAttribInterleavedBlocks, reg->sizeIdx));
-      fprintf(stdout, "    Rotation Angle: %d\n", rotateInt);
-      fprintf(stdout, "          Corner 0: (%0.1f, %0.1f)\n", p00.X, height - 1 - p00.Y);
-      fprintf(stdout, "          Corner 1: (%0.1f, %0.1f)\n", p10.X, height - 1 - p10.Y);
-      fprintf(stdout, "          Corner 2: (%0.1f, %0.1f)\n", p11.X, height - 1 - p11.Y);
-      fprintf(stdout, "          Corner 3: (%0.1f, %0.1f)\n", p01.X, height - 1 - p01.Y);
-      fprintf(stdout, "--------------------------------------------------\n");
-   }
-
-   if(opt->pageNumber)
-      fprintf(stdout, "%d:", pageIndex + 1);
-
-   if(opt->corners) {
-      fprintf(stdout, "%d,%d:", (int)(p00.X + 0.5), height - 1 - (int)(p00.Y + 0.5));
-      fprintf(stdout, "%d,%d:", (int)(p10.X + 0.5), height - 1 - (int)(p10.Y + 0.5));
-      fprintf(stdout, "%d,%d:", (int)(p11.X + 0.5), height - 1 - (int)(p11.Y + 0.5));
-      fprintf(stdout, "%d,%d:", (int)(p01.X + 0.5), height - 1 - (int)(p01.Y + 0.5));
-   }
-
-   if(opt->codewords) {
-      for(i = 0; i < msg->codeSize; i++) {
-         fprintf(stdout, "%c:%03d\n", (i < dataWordLength) ?
-               'd' : 'e', msg->code[i]);
-      }
-   }
-   else {
-      fwrite(msg->output, sizeof(char), msg->outputIdx, stdout);
-      if(opt->newline)
-         fputc('\n', stdout);
-   }
-   fflush(stdout);
-
-   return DMTX_SUCCESS;
-}
-
-/**
- *
- *
- */
-static void
-WriteDiagnosticImage(DmtxDecode *dec, DmtxRegion *reg, char *imagePath)
-{
-   int row, col;
-   int width, height;
-   int offset;
-   double shade;
-   FILE *fp;
-   DmtxRgb rgb;
-
-   fp = fopen(imagePath, "wb");
-   if(fp == NULL) {
-      perror(programName);
-      FatalError(EX_CANTCREAT, _("Unable to write image \"%s\""), imagePath);
-   }
-
-   width = dmtxImageGetProp(dec->image, DmtxPropScaledWidth);
-   height = dmtxImageGetProp(dec->image, DmtxPropScaledHeight);
-
-   /* Test each pixel of input image to see if it lies in region */
-   fprintf(fp, "P6\n%d %d\n255\n", width, height);
-   for(row = height - 1; row >= 0; row--) {
-      for(col = 0; col < width; col++) {
-
-         offset = dmtxImageGetOffset(dec->image, col, row);
-         if(offset == DMTX_BAD_OFFSET) {
-            rgb[0] = 0;
-            rgb[1] = 0;
-            rgb[2] = 128;
-         }
-         else {
-            dmtxImageGetRgb(dec->image, col, row, rgb);
-
-            if(dec->image->cache[offset] & 0x40) {
-               rgb[0] = 255;
-               rgb[1] = 0;
-               rgb[2] = 0;
-            }
-            else {
-               shade = (dec->image->cache[offset] & 0x80) ? 0.0 : 0.7;
-               rgb[0] += (shade * (255 - rgb[0]));
-               rgb[1] += (shade * (255 - rgb[1]));
-               rgb[2] += (shade * (255 - rgb[2]));
-            }
-         }
-         fwrite(rgb, sizeof(char), 3, fp);
-      }
-   }
-
-   fclose(fp);
-}
